@@ -1,6 +1,8 @@
 from dataclasses import asdict, replace
 
-from coiote.utils import ApiEndpoint, api_call
+from requests import Response
+
+from coiote.utils import ApiEndpoint, api_call, api_call_raw
 from coiote.v3.model.devices import *
 
 
@@ -10,12 +12,12 @@ class Devices(ApiEndpoint):
     ):
         super().__init__(*args, **kwargs, api_url="devices")
 
-    @api_call()
-    def get_all(self, search_criteria: Optional[str]) -> List[str]:
+    @api_call_raw
+    def get_all(self, search_criteria: Optional[str]) -> Response:
         return self.session.get(self.get_url(), params={"searchCriteria": search_criteria})
 
     @api_call(DevicesBatchAddResponse)
-    def create_batch(self, devices: List[Device]) -> DevicesBatchAddResponse:
+    def create_batch(self, devices: List[Device]) -> Response:
         updated_devices = self._update_endpoint_names(devices)
         return self.session.post(self.get_url("/batch"), json=[asdict(device) for device in updated_devices])
 
@@ -33,28 +35,46 @@ class Devices(ApiEndpoint):
         return [self._update_device(device) for device in devices]
 
     @api_call(DevicesFindResponse)
-    def get_device_details(self, search_criteria: Optional[str] = None, fields: List[str] = [], limit: int = 100,
-                           pageBookmark: Optional[str] = None) -> DevicesFindResponse:
-        return self.session.get(self.get_url("/find/details"), params={
+    def get_device_details(self,
+                           search_criteria: Optional[str] = None,
+                           fields: List[str] = None,
+                           limit: int = 100,
+                           page_bookmark: Optional[str] = None) -> Response:
+        if fields is None:
+            fields = []
+        params = {
             "searchCriteria": search_criteria,
             "fieldSelection": ",".join(fields),
             "limit": limit,
-            "pageBookmark": pageBookmark
-        })
+            "pageBookmark": page_bookmark
+        }
+        return self.session.get(self.get_url("/find/details"), params=params)
 
     @api_call(SingleDeviceAddResponse)
-    def create_one(self, device: Device) -> SingleDeviceAddResponse:
+    def create_one(self, device: Device) -> Response:
         updated = self._update_device(device)
         return self.session.post(self.get_url(), json=asdict(updated))
 
     @api_call(Device)
-    def get_one(self, device_id: str) -> Device:
+    def get_one(self, device_id: str) -> Response:
         return self.session.get(self.get_url(f"/{device_id}"))
 
-    @api_call()
-    def update_one(self, device_id, request: DeviceUpdateRequest) -> None:
+    def get_by_endpoint_name(self, endpoint_name: str) -> Optional[Device]:
+        find_response = self.get_device_details(
+            search_criteria=f"properties.endpointName eq '{endpoint_name}'",
+            fields=['id'],
+            limit=1
+        )
+        matching_devices = find_response.devices
+        if len(matching_devices) > 0:
+            return matching_devices[0]
+        else:
+            return None
+
+    @api_call_raw
+    def update_one(self, device_id, request: DeviceUpdateRequest) -> Response:
         return self.session.put(self.get_url(f"/{device_id}"), json=asdict(request))
 
-    @api_call()
-    def delete_one(self, device_id) -> None:
+    @api_call_raw
+    def delete_one(self, device_id) -> Response:
         return self.session.delete(self.get_url(f"/{device_id}"))
